#include <QApplication>
#include <QDebug>
#include <QHBoxLayout>
#include <QListWidget>
#include <QMainWindow>
#include <QOpenGLContext>
#include <QOpenGLFunctions>
#include <QOpenGLWindow>
#include <QWheelEvent>

static QString description() {
  QString t = QT_VERSION_STR;
#if QT_VERSION >= 0x050000
  t += QLatin1Char(' ') + QGuiApplication::platformName();
#endif
  return t;
}

class EventFilter : public QObject {
public:
  EventFilter(QObject *p) : QObject(p) {}

  bool eventFilter(QObject *watched, QEvent *event) {
	switch (event->type()) {
	case QEvent::Wheel:
	case QEvent::Enter:
	case QEvent::Leave:
	case QEvent::FocusIn:
	case QEvent::FocusOut:
	  qDebug() << watched << event;
	  break;
	default:
	  break;
	}
	return false;
  }
};

class OpenGLWindow : public QOpenGLWindow, public QOpenGLFunctions {
public:
  OpenGLWindow() : QOpenGLWindow(NoPartialUpdate) {}

protected:
  void initializeGL() override { initializeOpenGLFunctions(); }
  void wheelEvent(QWheelEvent *event) override;
  void paintGL() override;
};

void OpenGLWindow::wheelEvent(QWheelEvent *) { update(); }

void OpenGLWindow::paintGL() {
  QColor color;
  color.setRed(qrand() % 256);
  color.setGreen(qrand() % 256);
  color.setBlue(qrand() % 256);
  glClearColor(GLclampf(color.redF()), GLclampf(color.greenF()),
			   GLclampf(color.blueF()), GLclampf(color.alphaF()));

  glClear(GL_COLOR_BUFFER_BIT);
}

class MainWindow : public QMainWindow {
public:
  explicit MainWindow(QWidget *parent = 0);
};

MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent) {
  setObjectName("MainWindow");
  QWidget *cw = new QWidget(this);
  cw->setObjectName("Central");
  QHBoxLayout *horizontalLayout = new QHBoxLayout(cw);
  QListWidget *lw = new QListWidget(cw);
  lw->setObjectName("ListWidget");
  for (int i = 0; i < 50; ++i)
	lw->addItem("Item " + QString::number(i));
  horizontalLayout->addWidget(lw);

  OpenGLWindow *ow = new OpenGLWindow;
  ow->setObjectName("OpenGLWindow");
  QWidget *openglContainer = QWidget::createWindowContainer(ow, this);
  openglContainer->setObjectName("openglContainer");
  openglContainer->setMinimumSize(400, 200);
  openglContainer->setMouseTracking(true);
  openglContainer->setFocusPolicy(Qt::StrongFocus);
  horizontalLayout->addWidget(openglContainer);
  setCentralWidget(cw);
}

int main(int argc, char *argv[]) {
  QApplication a(argc, argv);
  a.installEventFilter(new EventFilter(&a));
  QString t = description();
  qDebug() << t;

  MainWindow w;
  w.setWindowTitle(t);
  w.show();

  return a.exec();
}
